@@ -24,6 +24,8 @@ import socket
 import unittest
 import warnings
 
+import io
+import gzip
 import json
 import mock
 import requests
@@ -214,6 +216,71 @@ class TestInfluxDBClient(unittest.TestCase):
                 m.last_request.body.decode('utf-8'),
             )
 
+    def test_write_gzip(self):
+        """Test write in TestInfluxDBClient object."""
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.POST,
+                "http://localhost:8086/write",
+                status_code=204
+            )
+
+            cli = InfluxDBClient(database='db', gzip=True)
+            cli.write(
+                {"database": "mydb",
+                 "retentionPolicy": "mypolicy",
+                 "points": [{"measurement": "cpu_load_short",
+                             "tags": {"host": "server01",
+                                      "region": "us-west"},
+                             "time": "2009-11-10T23:00:00Z",
+                             "fields": {"value": 0.64}}]}
+            )
+
+            compressed = io.BytesIO()
+            with gzip.GzipFile(
+                compresslevel=9,
+                fileobj=compressed,
+                mode='w'
+            ) as f:
+                f.write(
+                    b"cpu_load_short,host=server01,region=us-west "
+                    b"value=0.64 1257894000000000000\n"
+                )
+
+            self.assertEqual(
+                m.last_request.body,
+                compressed.getvalue(),
+            )
+
+    def test_write_points_gzip(self):
+        """Test write points for TestInfluxDBClient object."""
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.POST,
+                "http://localhost:8086/write",
+                status_code=204
+            )
+
+            cli = InfluxDBClient(database='db', gzip=True)
+            cli.write_points(
+                self.dummy_points,
+            )
+
+            compressed = io.BytesIO()
+            with gzip.GzipFile(
+                compresslevel=9,
+                fileobj=compressed,
+                mode='w'
+            ) as f:
+                f.write(
+                    b'cpu_load_short,host=server01,region=us-west '
+                    b'value=0.64 1257894000123456000\n'
+                )
+            self.assertEqual(
+                m.last_request.body,
+                compressed.getvalue(),
+            )
+
     def test_write_points_toplevel_attributes(self):
         """Test write points attrs for TestInfluxDBClient object."""
         with requests_mock.Mocker() as m:
@@ -265,6 +332,36 @@ class TestInfluxDBClient(unittest.TestCase):
         self.assertEqual(expected_last_body,
                          m.last_request.body.decode('utf-8'))
 
+    def test_write_points_batch_generator(self):
+        """Test write points batch from a generator for TestInfluxDBClient."""
+        dummy_points = [
+            {"measurement": "cpu_usage", "tags": {"unit": "percent"},
+             "time": "2009-11-10T23:00:00Z", "fields": {"value": 12.34}},
+            {"measurement": "network", "tags": {"direction": "in"},
+             "time": "2009-11-10T23:00:00Z", "fields": {"value": 123.00}},
+            {"measurement": "network", "tags": {"direction": "out"},
+             "time": "2009-11-10T23:00:00Z", "fields": {"value": 12.00}}
+        ]
+        dummy_points_generator = (point for point in dummy_points)
+        expected_last_body = (
+            "network,direction=out,host=server01,region=us-west "
+            "value=12.0 1257894000000000000\n"
+        )
+
+        with requests_mock.Mocker() as m:
+            m.register_uri(requests_mock.POST,
+                           "http://localhost:8086/write",
+                           status_code=204)
+            cli = InfluxDBClient(database='db')
+            cli.write_points(points=dummy_points_generator,
+                             database='db',
+                             tags={"host": "server01",
+                                   "region": "us-west"},
+                             batch_size=2)
+        self.assertEqual(m.call_count, 2)
+        self.assertEqual(expected_last_body,
+                         m.last_request.body.decode('utf-8'))
+
     def test_write_points_udp(self):
         """Test write points UDP for TestInfluxDBClient object."""
         s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
@@ -473,6 +570,29 @@ class TestInfluxDBClient(unittest.TestCase):
                 [{'value': 0.64, 'time': '2009-11-10T23:00:00Z'}]
             )
 
+    def test_query_msgpack(self):
+        """Test query method with a messagepack response."""
+        example_response = bytes(bytearray.fromhex(
+            "81a7726573756c74739182ac73746174656d656e745f696400a673657269"
+            "65739183a46e616d65a161a7636f6c756d6e7392a474696d65a176a67661"
+            "6c7565739192c70c05000000005d26178a019096c8cb3ff0000000000000"
+        ))
+
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.GET,
+                "http://localhost:8086/query",
+                request_headers={"Accept": "application/x-msgpack"},
+                headers={"Content-Type": "application/x-msgpack"},
+                content=example_response
+            )
+            rs = self.cli.query('select * from a')
+
+            self.assertListEqual(
+                list(rs.get_points()),
+                [{'v': 1.0, 'time': '2019-07-10T16:51:22.026253Z'}]
+            )
+
     def test_select_into_post(self):
         """Test SELECT.*INTO is POSTed."""
         example_response = (
@@ -666,6 +786,66 @@ class TestInfluxDBClient(unittest.TestCase):
                 [{'name': 'cpu'}, {'name': 'disk'}]
             )
 
+    def test_get_list_series(self):
+        """Test get a list of series from the database."""
+        data = {'results': [
+            {'series': [
+                {
+                    'values': [
+                        ['cpu_load_short,host=server01,region=us-west'],
+                        ['memory_usage,host=server02,region=us-east']],
+                    'columns': ['key']
+                }
+            ]}
+        ]}
+
+        with _mocked_session(self.cli, 'get', 200, json.dumps(data)):
+            self.assertListEqual(
+                self.cli.get_list_series(),
+                ['cpu_load_short,host=server01,region=us-west',
+                 'memory_usage,host=server02,region=us-east'])
+
+    def test_get_list_series_with_measurement(self):
+        """Test get a list of series from the database by filter."""
+        data = {'results': [
+            {'series': [
+                {
+                    'values': [
+                        ['cpu_load_short,host=server01,region=us-west']],
+                    'columns': ['key']
+                }
+            ]}
+        ]}
+
+        with _mocked_session(self.cli, 'get', 200, json.dumps(data)):
+            self.assertListEqual(
+                self.cli.get_list_series(measurement='cpu_load_short'),
+                ['cpu_load_short,host=server01,region=us-west'])
+
+    def test_get_list_series_with_tags(self):
+        """Test get a list of series from the database by tags."""
+        data = {'results': [
+            {'series': [
+                {
+                    'values': [
+                        ['cpu_load_short,host=server01,region=us-west']],
+                    'columns': ['key']
+                }
+            ]}
+        ]}
+
+        with _mocked_session(self.cli, 'get', 200, json.dumps(data)):
+            self.assertListEqual(
+                self.cli.get_list_series(tags={'region': 'us-west'}),
+                ['cpu_load_short,host=server01,region=us-west'])
+
+    @raises(Exception)
+    def test_get_list_series_fails(self):
+        """Test get a list of series from the database but fail."""
+        cli = InfluxDBClient('host', 8086, 'username', 'password')
+        with _mocked_session(cli, 'get', 401):
+            cli.get_list_series()
+
     def test_create_retention_policy_default(self):
         """Test create default ret policy for TestInfluxDBClient object."""
         example_response = '{"results":[{}]}'
@@ -706,6 +886,49 @@ class TestInfluxDBClient(unittest.TestCase):
                 '"db" duration 1d replication 4 shard duration 0s'
             )
 
+    def test_create_retention_policy_shard_duration(self):
+        """Test create retention policy with a custom shard duration."""
+        example_response = '{"results":[{}]}'
+
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.POST,
+                "http://localhost:8086/query",
+                text=example_response
+            )
+            self.cli.create_retention_policy(
+                'somename2', '1d', 4, database='db',
+                shard_duration='1h'
+            )
+
+            self.assertEqual(
+                m.last_request.qs['q'][0],
+                'create retention policy "somename2" on '
+                '"db" duration 1d replication 4 shard duration 1h'
+            )
+
+    def test_create_retention_policy_shard_duration_default(self):
+        """Test create retention policy with a default shard duration."""
+        example_response = '{"results":[{}]}'
+
+        with requests_mock.Mocker() as m:
+            m.register_uri(
+                requests_mock.POST,
+                "http://localhost:8086/query",
+                text=example_response
+            )
+            self.cli.create_retention_policy(
+                'somename3', '1d', 4, database='db',
+                shard_duration='1h', default=True
+            )
+
+            self.assertEqual(
+                m.last_request.qs['q'][0],
+                'create retention policy "somename3" on '
+                '"db" duration 1d replication 4 shard duration 1h '
+                'default'
+            )
+
     def test_alter_retention_policy(self):
         """Test alter retention policy for TestInfluxDBClient object."""
         example_response = '{"results":[{}]}'
@@ -1035,7 +1258,7 @@ class TestInfluxDBClient(unittest.TestCase):
             self.cli.revoke_privilege('', 'testdb', 'test')
 
     def test_get_list_privileges(self):
-        """Tst get list of privs for TestInfluxDBClient object."""
+        """Test get list of privs for TestInfluxDBClient object."""
         data = {'results': [
             {'series': [
                 {'columns': ['database', 'privilege'],
@@ -1177,16 +1400,11 @@ class TestInfluxDBClient(unittest.TestCase):
     def test_chunked_response(self):
         """Test chunked reponse for TestInfluxDBClient object."""
         example_response = \
-            u'{"results":[{"statement_id":0,"series":' \
-            '[{"name":"cpu","columns":["fieldKey","fieldType"],"values":' \
-            '[["value","integer"]]}],"partial":true}]}\n{"results":' \
-            '[{"statement_id":0,"series":[{"name":"iops","columns":' \
-            '["fieldKey","fieldType"],"values":[["value","integer"]]}],' \
-            '"partial":true}]}\n{"results":[{"statement_id":0,"series":' \
-            '[{"name":"load","columns":["fieldKey","fieldType"],"values":' \
-            '[["value","integer"]]}],"partial":true}]}\n{"results":' \
-            '[{"statement_id":0,"series":[{"name":"memory","columns":' \
-            '["fieldKey","fieldType"],"values":[["value","integer"]]}]}]}\n'
+            u'{"results":[{"statement_id":0,"series":[{"columns":["key"],' \
+            '"values":[["cpu"],["memory"],["iops"],["network"]],"partial":' \
+            'true}],"partial":true}]}\n{"results":[{"statement_id":0,' \
+            '"series":[{"columns":["key"],"values":[["qps"],["uptime"],' \
+            '["df"],["mount"]]}]}]}\n'
 
         with requests_mock.Mocker() as m:
             m.register_uri(
@@ -1194,23 +1412,20 @@ class TestInfluxDBClient(unittest.TestCase):
                 "http://localhost:8086/query",
                 text=example_response
             )
-            response = self.cli.query('show series limit 4 offset 0',
+            response = self.cli.query('show series',
                                       chunked=True, chunk_size=4)
-            self.assertTrue(len(response) == 4)
-            self.assertEqual(response.__repr__(), ResultSet(
-                {'series': [{'values': [['value', 'integer']],
-                             'name': 'cpu',
-                             'columns': ['fieldKey', 'fieldType']},
-                            {'values': [['value', 'integer']],
-                             'name': 'iops',
-                             'columns': ['fieldKey', 'fieldType']},
-                            {'values': [['value', 'integer']],
-                             'name': 'load',
-                             'columns': ['fieldKey', 'fieldType']},
-                            {'values': [['value', 'integer']],
-                             'name': 'memory',
-                             'columns': ['fieldKey', 'fieldType']}]}
-            ).__repr__())
+            res = list(response)
+            self.assertTrue(len(res) == 2)
+            self.assertEqual(res[0].__repr__(), ResultSet(
+                {'series': [{
+                    'columns': ['key'],
+                    'values': [['cpu'], ['memory'], ['iops'], ['network']]
+                }]}).__repr__())
+            self.assertEqual(res[1].__repr__(), ResultSet(
+                {'series': [{
+                    'columns': ['key'],
+                    'values': [['qps'], ['uptime'], ['df'], ['mount']]
+                }]}).__repr__())
 
 
 class FakeClient(InfluxDBClient):
